/* bots.c - NeonServ v5.6
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */

#include "bots.h"
#include "timeq.h"
#include "mysqlConn.h"
#include "ClientSocket.h"
#include "UserNode.h"
#include "ChanNode.h"
#include "ChanUser.h"
#include "version.h"
#include "modcmd.h"
#include "DBHelper.h"
#include "IRCEvents.h"
#include "tools.h"
#include "log.h"

struct cmd_bot_alias {
    int botid;
    char *alias;
    struct cmd_bot_alias *next;
};

static struct cmd_bot_alias *bot_aliases = NULL;

static void start_zero_bots() {
    struct ClientSocket *client;
    MYSQL_RES *res, *res2;
    MYSQL_ROW row;
    
    printf_mysql_query("SELECT `nick`, `ident`, `realname`, `server`, `port`, `pass`, `textbot`, `id`, `queue`, `ssl`, `bind`, `secret` FROM `bots` WHERE `botclass` = '0' AND `active` = '1'");
    res = mysql_use();
    
    while ((row = mysql_fetch_row(res)) != NULL) {
        client = create_socket(row[3], atoi(row[4]), row[10], row[5], row[0], row[1], row[2]);
        client->flags |= (strcmp(row[6], "0") ? SOCKET_FLAG_PREFERRED : 0);
        client->flags |= (strcmp(row[8], "0") ? SOCKET_FLAG_USE_QUEUE : 0);
        client->flags |= (strcmp(row[9], "0") ? SOCKET_FLAG_SSL : 0);
        client->flags |= (strcmp(row[11], "0") ? SOCKET_FLAG_SECRET_BOT : 0);
        client->botid = 0;
        client->clientid = atoi(row[7]);
        connect_socket(client);
        
        printf_mysql_query("SELECT `command`, `function`, `parameters`, `global_access`, `chan_access`, `flags` FROM `bot_binds` WHERE `botclass` = '0' AND `botid` = '%d'", client->clientid);
        res2 = mysql_use();
        while ((row = mysql_fetch_row(res2)) != NULL) {
            if(bind_botwise_cmd_to_command(0, client->clientid, row[0], row[1])) {
                if(row[2] && strcmp(row[2], "")) {
                    bind_botwise_set_parameters(0, client->clientid, row[0], row[2]);
                }
                if(row[3]) {
                    bind_botwise_set_global_access(0, client->clientid, row[0], atoi(row[3]));
                }
                if(row[4]) {
                    bind_botwise_set_channel_access(0, client->clientid, row[0], row[4]);
                }
                if(strcmp(row[5], "0"))
                    bind_botwise_set_bind_flags(0, client->clientid, row[0], atoi(row[5]));
            }
        }
        bind_botwise_unbound_required_functions(0, client->clientid);
    }
}

static void zero_bots_trigger_callback(int clientid, struct ChanNode *chan, char *trigger) {
    MYSQL_RES *res;
    MYSQL_ROW row;
    loadChannelSettings(chan);
    if(!(chan->flags & CHANFLAG_CHAN_REGISTERED)) {
        strcpy(trigger, "+");
        return;
    }
    printf_mysql_query("SELECT `trigger`, `defaulttrigger` FROM `bot_channels` LEFT JOIN `bots` ON `botid` = `bots`.`id` WHERE `chanid` = '%d' AND `botclass` = '0' AND `botid` = '%d'", chan->channel_id, clientid);
    res = mysql_use();
    if(!(row = mysql_fetch_row(res))) {
        strcpy(trigger, "");
        return;
    }
    if(row[0] && *row[0])
        strcpy(trigger, row[0]);
    else
        strcpy(trigger, ((row[1] && *row[1]) ? row[1] : "+"));
}

static void zero_bots_bot_ready(struct ClientSocket *client) {
    if(client->botid != 0)
        return;
    MYSQL_RES *res;
    MYSQL_ROW row;
    
    printf_mysql_query("SELECT `automodes`, `oper_user`, `oper_pass` FROM `bots` WHERE `id` = '%d'", client->clientid);
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) != NULL) {
        if(row[1] && row[2]) {
            putsock(client, "OPER %s %s", row[1], row[2]);
        }
        putsock(client, "MODE %s +%s", client->user->nick, row[0]);
    }
    
    printf_mysql_query("SELECT `channel_name`, `channel_key` FROM `bot_channels` LEFT JOIN `channels` ON `chanid` = `channel_id` WHERE `botid` = '%d' AND `suspended` = '0'", client->clientid);
    res = mysql_use();
    
    while ((row = mysql_fetch_row(res)) != NULL) {
        putsock(client, "JOIN %s %s", row[0], row[1]);
    }
}

static TIMEQ_CALLBACK(load_timed_bans) {
    MYSQL_RES *res;
    MYSQL_ROW row;
    //load all timed bans for the next 7 days
    printf_mysql_query("SELECT `ban_id`, `ban_timeout` FROM `bans` WHERE `ban_timeout` > 0 AND `ban_timeout` < (UNIX_TIMESTAMP() + (86400 * 7))");
    res = mysql_use();
    char nameBuf[20];
    while ((row = mysql_fetch_row(res)) != NULL) {
        if(atol(row[1]) - time(0) > 0) {
            sprintf(nameBuf, "ban_%s", row[0]);
            timeq_add_name(nameBuf, atol(row[1]) - time(0), 0, channel_ban_timeout, strdup(row[0]));
        } else {
            //timed out
            printf_mysql_query("DELETE FROM `bans` WHERE `ban_id` = '%s'", row[0]);
        }
    }
    timeq_add(86400*7, 0, load_timed_bans, NULL);
}

void init_bots() {
    set_bot_alias(0, "0");
    start_zero_bots();
    set_trigger_callback(0, 0, zero_bots_trigger_callback);
	bind_bot_ready(zero_bots_bot_ready, 0);
    
    timeq_add(10, 0, load_timed_bans, NULL);
    
}

struct ClientSocket *getChannelBot(struct ChanNode *chan, int botid) {
    struct ClientSocket *bot, *use_bot = NULL, *second_bot = NULL, *third_bot = NULL;
    struct ChanUser *chanuser;
    for(bot = getBots(SOCKET_FLAG_READY, NULL); bot; bot = getBots(SOCKET_FLAG_READY, bot)) {
        if(botid && bot->botid != botid) continue;
        if(!chan || (chanuser = getChanUser(bot->user, chan)) != NULL) {
            if(chan && (chanuser->flags & CHANUSERFLAG_OPPED)) {
                use_bot = bot;
                if(bot->flags & SOCKET_FLAG_PREFERRED) break;
            } else if(bot->flags & SOCKET_FLAG_PREFERRED)
                second_bot = bot;
            else
                third_bot = bot;
        }
    }
    if(!use_bot) use_bot = second_bot;
    if(!use_bot) use_bot = third_bot;
    return use_bot;
}

void requestOp(struct UserNode *user, struct ChanNode *chan) {
    struct ClientSocket *bot, *userbot = NULL;
    struct ChanUser *chanuser = getChanUser(user, chan);
    char opped = 0;
    if(!chanuser) return;
    if((chanuser->flags & CHANUSERFLAG_OPPED)) return;
    for(bot = getBots(SOCKET_FLAG_READY, NULL); bot; bot = getBots(SOCKET_FLAG_READY, bot)) {
        if(!opped && (chanuser = getChanUser(bot->user, chan)) != NULL && (chanuser->flags & CHANUSERFLAG_OPPED)) {
            opped = 1;
            putsock(bot, "MODE %s +o %s", chan->name, user->nick);
        }
        if(bot->user == user) {
            userbot = bot;
        }
    }
    if(!opped) {
        if(userbot && (isUserModeSet(user, 'o') || isUserModeSet(user, 'O') || isUserModeSet(user, 'k') || isUserModeSet(user, 'X'))) {
            putsock(userbot, "MODE %s +o %s", chan->name, user->nick);
            putsock(userbot, "OPMODE %s +o %s", chan->name, user->nick);
        }
    }
}

void requestInvite(struct UserNode *user, struct ChanNode *chan) {
    struct ClientSocket *bot;
    struct ChanUser *chanuser = getChanUser(user, chan);
    char invited = 0;
    if(chanuser) return;
    for(bot = getBots(SOCKET_FLAG_READY, NULL); bot; bot = getBots(SOCKET_FLAG_READY, bot)) {
        if(!invited && (chanuser = getChanUser(bot->user, chan)) != NULL && (chanuser->flags & CHANUSERFLAG_OPPED)) {
            invited = 1;
            putsock(bot, "INVITE %s %s", user->nick, chan->name);
        }
    }
}

TIMEQ_CALLBACK(channel_ban_timeout) {
    char *str_banid = data;
    MYSQL_RES *res;
    MYSQL_ROW row;
    printf_mysql_query("SELECT `ban_mask`, `channel_name` FROM `bans` LEFT JOIN `channels` ON `ban_channel` = `channel_id` WHERE `ban_id` = '%s'", str_banid);
    res = mysql_use();
    struct ChanNode *chan;
    if((row = mysql_fetch_row(res)) != NULL && (chan = getChanByName(row[1])) != NULL) {
        struct ClientSocket *use_bot = getChannelBot(chan, 0);
        if(use_bot) {
            putsock(use_bot, "MODE %s -b %s", chan->name, row[0]);
        }
        printf_mysql_query("DELETE FROM `bans` WHERE `ban_id` = '%s'", str_banid);
    }
    free(str_banid);
}

static int general_ctcp(char *buffer, char *command, char *text);

void general_event_privctcp(struct UserNode *user, struct UserNode *target, char *command, char *text) {
    char ctcpBuf[MAXLEN];
    if(general_ctcp(ctcpBuf, command, text)) {
        struct ClientSocket *bot;
        for(bot = getBots(SOCKET_FLAG_READY, NULL); bot; bot = getBots(SOCKET_FLAG_READY, bot)) {
            if(bot->user == target) break;
        }
        if(bot)
            putsock(bot, "NOTICE %s :\001%s\001", user->nick, ctcpBuf);
    }
}

static int general_ctcp(char *buffer, char *command, char *text) {
    if(!stricmp(command, "VERSION")) {
        sprintf(buffer, "VERSION NeonServ v%s.%d by pk910 (%s)", NEONSERV_VERSION, patchlevel, (strcmp(revision, "") ? revision : "-"));
        return 1;
    }
    if(!stricmp(command, "FINGER")) {
        sprintf(buffer, "FINGER NeonServ v%s.%d (%s) build %s lines C code using " COMPILER " (see +netinfo)", NEONSERV_VERSION, patchlevel, (strcmp(revision, "") ? revision : "-"), codelines);
        return 1;
    }
    if(!stricmp(command, "PING")) {
        sprintf(buffer, "PING %s", (text ? text : "0"));
        return 1;
    }
    if(!stricmp(command, "TIME")) {
        time_t rawtime;
        struct tm *timeinfo;
        char timeBuf[80];
        time(&rawtime);
        timeinfo = localtime(&rawtime);
        strftime(timeBuf, 80, "%c", timeinfo);
        sprintf(buffer, "TIME %s", timeBuf);
        return 1;
    }
    return 0;
}

void set_bot_alias(int botid, char *alias) {
    struct cmd_bot_alias *botalias, *existing_alias = NULL;
    for(botalias = bot_aliases; botalias; botalias = botalias->next) {
        if(!stricmp(botalias->alias, alias))
            return;
        if(botalias->botid == botid)
            existing_alias = botalias;
    }
    if(existing_alias) {
        free(existing_alias->alias);
        existing_alias->alias = strdup(alias);
        return;
    }
    botalias = malloc(sizeof(*botalias));
    if (!botalias) {
        printf_log("main", LOG_ERROR, "%s:%d malloc() failed", __FILE__, __LINE__);
        return;
    }
    botalias->botid = botid;
    botalias->alias = strdup(alias);
    botalias->next = bot_aliases;
    bot_aliases = botalias;
}

const char *resolve_botid(int botid) {
    struct cmd_bot_alias *botalias;
    for(botalias = bot_aliases; botalias; botalias = botalias->next) {
        if(botalias->botid == botid)
            return botalias->alias;
    }
    return NULL;
}

int resolve_botalias(const char *alias) {
    struct cmd_bot_alias *botalias;
    for(botalias = bot_aliases; botalias; botalias = botalias->next) {
        if(!stricmp(botalias->alias, alias))
            return botalias->botid;
    }
    return -1;
}
