/* bot_NeonBackup.c - NeonServ v5.6
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */
#include "../module.h"
#include "../botid.h"

#include "bot_NeonBackup.h"
#include "../../modcmd.h"
#include "cmd_neonbackup.h"
#include "../../lang.h"
#include "../../mysqlConn.h"
#include "../../ClientSocket.h"
#include "../../UserNode.h"
#include "../../ChanNode.h"
#include "../../ChanUser.h"
#include "../../IRCEvents.h"
#include "../../IRCParser.h"
#include "../../bots.h"
#include "../../DBHelper.h"
#include "../../WHOHandler.h"

#define BOTID NEONBACKUP_BOTID
#define BOTALIAS "NeonBackup"

static const struct default_language_entry msgtab[] = {
    {NULL, NULL}
};

static void neonbackup_bot_ready(struct ClientSocket *client) {
    if(client->botid != BOTID)
        return;
    MYSQL_RES *res;
    MYSQL_ROW row;
    
    printf_mysql_query("SELECT `automodes`, `oper_user`, `oper_pass` FROM `bots` WHERE `id` = '%d'", client->clientid);
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) != NULL) {
        if(row[1] && row[2]) {
            putsock(client, "OPER %s %s", row[1], row[2]);
        }
        putsock(client, "MODE %s +%s", client->user->nick, row[0]);
    }
    
    printf_mysql_query("SELECT `channel_name`, `channel_key` FROM `bot_channels` LEFT JOIN `channels` ON `chanid` = `channel_id` WHERE `botid` = '%d' AND `suspended` = '0'", client->clientid);
    res = mysql_use();
    
    while ((row = mysql_fetch_row(res)) != NULL) {
        putsock(client, "JOIN %s %s", row[0], row[1]);
    }
}

static void neonbackup_trigger_callback(int clientid, struct ChanNode *chan, char *trigger) {
    MYSQL_RES *res;
    MYSQL_ROW row;
    loadChannelSettings(chan);
    if(!(chan->flags & CHANFLAG_CHAN_REGISTERED)) {
        strcpy(trigger, "!");
        return;
    }
    printf_mysql_query("SELECT `trigger`, `defaulttrigger` FROM `bot_channels` LEFT JOIN `bots` ON `botid` = `bots`.`id` WHERE `chanid` = '%d' AND `botclass` = '%d'", chan->channel_id, BOTID);
    res = mysql_use();
    if(!(row = mysql_fetch_row(res))) {
        strcpy(trigger, "+");
        return;
    }
    if(row[0] && *row[0])
        strcpy(trigger, row[0]);
    else
        strcpy(trigger, ((row[1] && *row[1]) ? row[1] : "+"));
}

static void start_bots(int type) {
    struct ClientSocket *client;
    MYSQL_RES *res, *res2;
    MYSQL_ROW row;
    
    if(type == MODSTATE_STARTSTOP) {
        printf_mysql_query("SELECT `nick`, `ident`, `realname`, `server`, `port`, `pass`, `textbot`, `id`, `queue`, `ssl`, `bind`, `secret` FROM `bots` WHERE `botclass` = '%d' AND `active` = '1'", BOTID);
        res = mysql_use();
        
        while ((row = mysql_fetch_row(res)) != NULL) {
            client = create_socket(row[3], atoi(row[4]), row[10], row[5], row[0], row[1], row[2]);
            client->flags |= (strcmp(row[6], "0") ? SOCKET_FLAG_PREFERRED : 0);
            client->flags |= (strcmp(row[8], "0") ? SOCKET_FLAG_USE_QUEUE : 0);
            client->flags |= (strcmp(row[9], "0") ? SOCKET_FLAG_SSL : 0);
            client->flags |= (strcmp(row[11], "0") ? SOCKET_FLAG_SECRET_BOT : 0);
            client->flags |= SOCKET_FLAG_REQUEST_INVITE | SOCKET_FLAG_REQUEST_OP;
            client->botid = BOTID;
            client->clientid = atoi(row[7]);
            connect_socket(client);
        }
    }
    
    printf_mysql_query("SELECT `command`, `function`, `parameters`, `global_access`, `chan_access`, `flags` FROM `bot_binds` WHERE `botclass` = '%d'", BOTID);
    res2 = mysql_use();
    while ((row = mysql_fetch_row(res2)) != NULL) {
        if(bind_cmd_to_command(BOTID, row[0], row[1])) {
            if(row[2] && strcmp(row[2], "")) {
                bind_set_parameters(BOTID, row[0], row[2]);
            }
            if(row[3]) {
                bind_set_global_access(BOTID, row[0], atoi(row[3]));
            }
            if(row[4]) {
                bind_set_channel_access(BOTID, row[0], row[4]);
            }
            if(strcmp(row[5], "0"))
                bind_set_bind_flags(BOTID, row[0], atoi(row[5]));
        }
    }
    bind_unbound_required_functions(BOTID);
}

void neonbackup_recover_chan(struct ChanNode *chan) {
    struct ClientSocket *bot = getChannelBot(chan, BOTID); // prefer backup bot ;)
    struct ChanUser *chanuser = (bot ? getChanUser(bot->user, chan) : NULL);
    if(!chanuser || !(chanuser->flags & CHANUSERFLAG_OPPED)) {
        //search an opped bot
        for(bot = getBots(SOCKET_FLAG_READY, NULL); bot; bot = getBots(SOCKET_FLAG_READY, bot)) {
            if((chanuser = getChanUser(bot->user, chan)) && (chanuser->flags & CHANUSERFLAG_OPPED))
                break;
        }
    }
    if(!bot) //no opped bots present... channel can't be recovered
        return;
    struct ClientSocket *target;
    struct ModeBuffer *modeBuf;
    modeBuf = initModeBuffer(bot, chan);
    for(target = getBots(SOCKET_FLAG_READY, NULL); target; target = getBots(SOCKET_FLAG_READY, target)) {
        if((target->flags & SOCKET_FLAG_REQUEST_OP) && (chanuser = getChanUser(target->user, chan)) && !(chanuser->flags & CHANUSERFLAG_OPPED)) {
            modeBufferOp(modeBuf, target->user->nick);
        }
    }
    freeModeBuffer(modeBuf);
}

static void neonbackup_event_join(struct ChanUser *chanuser) {
    struct ClientSocket *client = getChannelBot(chanuser->chan, BOTID);
    if(!client) return; //we can't "see" this event
    if(chanuser->user == client->user) {
        requestOp(client->user, chanuser->chan);
        neonbackup_recover_chan(chanuser->chan);
    }
}

static void neonbackup_event_invite(struct ClientSocket *client, struct UserNode *user, char *channel) {
	if(client->botid != BOTID)
		return;
    MYSQL_RES *res;
    MYSQL_ROW row;
    printf_mysql_query("SELECT `botid`, `bot_channels`.`id`, `suspended` FROM `bot_channels` LEFT JOIN `bots` ON `bot_channels`.`botid` = `bots`.`id` LEFT JOIN `channels` ON `chanid` = `channel_id` WHERE `channel_name` = '%s' AND `botclass` = '%d'", escape_string(channel), client->botid);
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) == NULL) {
        reply(client, user, "NS_INVITE_FAIL", channel, client->user->nick);
        return;
    }
    if(!strcmp(row[2], "1")) {
        reply(client, user, "MODCMD_CHAN_SUSPENDED");
        return;
    }
    int botid = atoi(row[0]);
    struct ClientSocket *bot;
    for(bot = getBots(SOCKET_FLAG_READY, NULL); bot; bot = getBots(SOCKET_FLAG_READY, bot)) {
        if(bot->clientid == botid)
            break;
    }
    if(bot) {
        struct ChanNode *chan = getChanByName(channel);
        if(chan && isUserOnChan(bot->user, chan)) {
            reply(client, user, "NS_INVITE_ON_CHAN", bot->user->nick, chan->name);
        } else
            putsock(bot, "JOIN %s", channel);
    }
}

void init_NeonBackup(int type) {
    set_bot_alias(BOTID, BOTALIAS);
    start_bots(type);
    
    if(type == MODSTATE_REBIND) return;
    
    //register events
    bind_bot_ready(neonbackup_bot_ready, module_id);
    bind_join(neonbackup_event_join, module_id);
	bind_invite(neonbackup_event_invite, module_id);
    
    set_trigger_callback(BOTID, module_id, neonbackup_trigger_callback);
    
    register_default_language_table(msgtab);
}

void free_NeonBackup(int type) {
    unbind_allcmd(BOTID);
    if(type == MODSTATE_STARTSTOP) {
        //disconnect all our bots
        struct ClientSocket *client;
        for(client = getBots(0, NULL); client; client = getBots(0, client)) {
            if(client->botid == BOTID) {
                unbind_botwise_allcmd(0, client->clientid);
                close_socket(client);
                break;
            }
        }
    }
}

#undef BOTID
#undef BOTALIAS
