/* cmd_neonserv_giveowner.c - NeonServ v5.6
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */

#include "cmd_neonserv.h"

/*
* argv[0] - nick / *auth
* argv[1] - key
*/

#define GIVEOWNER_TIMEOUT 86400 /* 60*60*24 = 86400 */

static USERAUTH_CALLBACK(neonserv_cmd_giveowner_nick_lookup);
static void neonserv_cmd_giveowner_async1(struct ClientSocket *client, struct ClientSocket *textclient, struct UserNode *user, struct ChanNode *chan, struct Event *event, char *nick, char *auth, char *key);

struct neonserv_cmd_giveowner_cache {
    struct ClientSocket *client, *textclient;
    struct UserNode *user;
    struct ChanNode *chan;
    struct Event *event;
    char *nick;
    char *key;
};

CMD_BIND(neonserv_cmd_giveowner) {
    MYSQL_RES *res;
    MYSQL_ROW row;
    printf_mysql_query("SELECT `channel_lastgiveowner` FROM `channels` WHERE `channel_id` = '%d'", chan->channel_id);
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) == NULL) return;
    if(strcmp(row[0], "0") && (atoi(row[0]) + GIVEOWNER_TIMEOUT) > time(0)) {
        char timeBuf[MAXLEN];
        reply(textclient, user, "NS_GIVEOWNER_TIMEOUT", timeToStr(user, (GIVEOWNER_TIMEOUT - (time(0) - atoi(row[0]))), 2, timeBuf), chan->name);
        return;
    }
    if(argv[0][0] == '*') {
        //we've got an auth
        argv[0]++;
        neonserv_cmd_giveowner_async1(client, textclient, user, chan, event, argv[0], argv[0], (argc != 1 ? argv[1] : NULL));
    } else {
        struct UserNode *cuser = getUserByNick(argv[0]);
        if(!cuser) {
            cuser = createTempUser(argv[0]);
			if(!cuser) {
                reply(textclient, user, "NS_USER_UNKNOWN", argv[0]);
                return;
            }
            cuser->flags |= USERFLAG_ISTMPUSER;
        }
        if(cuser->flags & USERFLAG_ISAUTHED) {
            neonserv_cmd_giveowner_async1(client, textclient, user, chan, event, argv[0], cuser->auth, (argc != 1 ? argv[1] : NULL));
        } else {
            struct neonserv_cmd_giveowner_cache *cache = malloc(sizeof(*cache));
            if (!cache) {
                printf_log("neonserv", LOG_ERROR, "%s:%d malloc() failed", __FILE__, __LINE__);
                return;
            }
            cache->client = client;
            cache->textclient = textclient;
            cache->user = user;
            cache->chan = chan;
            cache->event = event;
            cache->nick = strdup(argv[0]);
            cache->key = (argc != 1 ? strdup(argv[1]) : NULL);
            get_userauth(cuser, module_id, neonserv_cmd_giveowner_nick_lookup, cache);
        }
    }
}

static USERAUTH_CALLBACK(neonserv_cmd_giveowner_nick_lookup) {
    struct neonserv_cmd_giveowner_cache *cache = data;
    if(!user) {
        //USER_DOES_NOT_EXIST
        reply(cache->textclient, cache->user, "NS_USER_UNKNOWN", cache->nick);
    }
    else if(!(user->flags & USERFLAG_ISAUTHED)) {
        //USER_NOT_AUTHED
        reply(cache->textclient, cache->user, "NS_USER_NEED_AUTH", cache->nick);
    }
    else
        neonserv_cmd_giveowner_async1(cache->client, cache->textclient, cache->user, cache->chan, cache->event, user->nick, user->auth, cache->key);
    free(cache->nick);
    if(cache->key)
        free(cache->key);
    free(cache);
}

static void neonserv_cmd_giveowner_async1(struct ClientSocket *client, struct ClientSocket *textclient, struct UserNode *user, struct ChanNode *chan, struct Event *event, char *nick, char *auth, char *key) {
    //we've got a valid auth now...
    if(!stricmp(user->auth, auth)) {
        reply(textclient, user, "NS_GIVEOWNER_SELF");
        return;
    }
    MYSQL_RES *res;
    MYSQL_ROW row;
    printf_mysql_query("SELECT `user_user`, `dnr_timeout`, `dnr_reason`, `dnr_id` FROM `donotregister` LEFT JOIN `users` ON `dnr_user` = `user_id` WHERE `dnr_target` = '%s'", escape_string(auth));
    res = mysql_use();
    if((row = mysql_fetch_row(res)) != NULL) {
        int expire_time = atoi(row[1]);
        if(expire_time) {
            if(expire_time - time(0) <= 0) {
                printf_mysql_query("DELETE FROM `donotregister` WHERE `dnr_id` = '%s'", row[3]);
            } else {
                reply(textclient, user, "NS_DNR_SET_ANONYM", auth);
                return;
            }
        } else {
            reply(textclient, user, "NS_DNR_SET_ANONYM", auth);
            return;
        }
    }
    int userid;
    printf_mysql_query("SELECT `user_id` FROM `users` WHERE `user_user` = '%s'", escape_string(auth));
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) != NULL) {
        userid = atoi(row[0]);
        //check if the user is already added
        printf_mysql_query("SELECT `chanuser_access`, `chanuser_id` FROM `chanusers` WHERE `chanuser_cid` = '%d' AND `chanuser_uid` = '%d'", chan->channel_id, userid);
        res = mysql_use();
        if ((row = mysql_fetch_row(res)) != NULL) {
            int seed = 0;
            char *tmp;
            char giveownerkey[16];
            for(tmp = user->auth; *tmp; tmp++)
                seed = (seed * 0xEECE66DL ^ ((*tmp << 24) | (*tmp << 16) | (*tmp << 8) | *tmp));
            for(tmp = chan->name; *tmp; tmp++)
                seed = (seed * 0xEECE66DL ^ ((*tmp << 24) | (*tmp << 16) | (*tmp << 8) | *tmp));
            sprintf(giveownerkey, "%08x", seed);
            if(key && !stricmp(giveownerkey, key)) {
                //give ownership
                printf_mysql_query("UPDATE `chanusers` SET `chanuser_access` = '500' WHERE `chanuser_id` = '%s'", row[1]);
                printf_mysql_query("UPDATE `chanusers` SET `chanuser_access` = '499' WHERE `chanuser_cid` = '%d' AND `chanuser_uid` = (SELECT `user_id` FROM `users` WHERE `user_user` = '%s')", chan->channel_id, escape_string(user->auth));
                printf_mysql_query("INSERT INTO `owner_history` (`owner_history_cid`, `owner_history_from_uid`, `owner_history_to_uid`, `owner_history_time`) VALUE ('%d', (SELECT `user_id` FROM `users` WHERE `user_user` = '%s'), '%d', UNIX_TIMESTAMP())", chan->channel_id, escape_string(user->auth), userid);
                reply(textclient, user, "NS_GIVEOWNER_DONE", chan->name, auth);
                logEvent(event);
            } else {
                reply(textclient, user, "NS_GIVEOWNER_CONFIRM", auth, giveownerkey);
            }
            return;
        }
    }
    reply(textclient, user, "NS_NOT_ON_USERLIST", nick, chan->name);
}
