/* cmd_neonserv_search.c - NeonServ v5.6
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */

#include "cmd_neonserv.h"

#define CMD_SEARCH_FLAG_HAS_NODELETE 0x001
#define CMD_SEARCH_FLAG_NOT_NODELETE 0x002
#define CMD_SEARCH_FLAG_HAS_SUSPENDED 0x004
#define CMD_SEARCH_FLAG_NOT_SUSPENDED 0x008
#define CMD_SEARCH_FLAG_IS_JOINED  0x010 /* state */
#define CMD_SEARCH_FLAG_NOT_JOINED 0x020 /* state */
#define CMD_SEARCH_FLAG_IS_OPPED   0x040 /* state */
#define CMD_SEARCH_FLAG_NOT_OPPED  0x080 /* state */
#define CMD_SEARCH_FLAG_IS_VOICED  0x100 /* state */
#define CMD_SEARCH_FLAG_NOT_VOICED 0x200 /* state */
#define CMD_SEARCH_FLAG_IS_ZOMBIE  0x400
#define CMD_SEARCH_FLAG_NOT_ZOMBIE 0x800

#define CMD_SEARCH_FLAG_STATES     0x3f0

struct neonserv_cmd_search_criteria {
    char *name;
    char *registrar;
    char *onchan;
    unsigned int flags : 16;
    unsigned int unvisited;
    unsigned int registered;
    unsigned int limit : 16;
};

static char neonserv_cmd_search_zombie = 0;

CMD_BIND(neonserv_cmd_search) {
    //ok parse the criterias
    struct neonserv_cmd_search_criteria criteria;
    memset(&criteria, 0, sizeof(criteria));
    criteria.limit = 50;
    int i, show_chans = 0, positive;
    if(!stricmp(argv[0], "print")) {
        show_chans = 1;
    }
    for(i = 1; i < argc; i += 2) {
        if(argc <= i+1) {
            reply(textclient, user, "MODCMD_LESS_PARAM_COUNT");
            return;
        }
        if(!stricmp(argv[i], "name")) criteria.name = argv[i+1];
        else if(!stricmp(argv[i], "registrar")) criteria.registrar = argv[i+1];
        else if(!stricmp(argv[i], "onchan")) criteria.onchan = argv[i+1];
        else if(!stricmp(argv[i], "unvisited")) criteria.unvisited = strToTime(user, argv[i+1]);
        else if(!stricmp(argv[i], "registered")) criteria.registered = strToTime(user, argv[i+1]);
        else if(!stricmp(argv[i], "flags")) {
            if(argv[i+1][0] == '+') {
                positive = 1;
                argv[i+1]++;
            } else if(argv[i+1][0] == '-') {
                positive = 0;
                argv[i+1]++;
            } else
                positive = 1;
            if(!stricmp(argv[i+1], "nodelete")) {
                if(positive)
                    criteria.flags |= CMD_SEARCH_FLAG_HAS_NODELETE;
                else
                    criteria.flags |= CMD_SEARCH_FLAG_NOT_NODELETE;
            } else if(!stricmp(argv[i+1], "suspended")) {
                if(positive)
                    criteria.flags |= CMD_SEARCH_FLAG_HAS_SUSPENDED;
                else
                    criteria.flags |= CMD_SEARCH_FLAG_NOT_SUSPENDED;
            } else if(!stricmp(argv[i+1], "zombie")) {
                if(positive)
                    criteria.flags |= CMD_SEARCH_FLAG_IS_ZOMBIE;
                else
                    criteria.flags |= CMD_SEARCH_FLAG_NOT_ZOMBIE;
            }
        }
        else if(!stricmp(argv[i], "state")) {
            if(argv[i+1][0] == '+') {
                positive = 1;
                argv[i+1]++;
            } else if(argv[i+1][0] == '-') {
                positive = 0;
                argv[i+1]++;
            } else
                positive = 1;
            if(!stricmp(argv[i+1], "joined")) {
                if(positive)
                    criteria.flags |= CMD_SEARCH_FLAG_IS_JOINED;
                else
                    criteria.flags |= CMD_SEARCH_FLAG_NOT_JOINED;
            } else if(!stricmp(argv[i+1], "opped")) {
                if(positive)
                    criteria.flags |= CMD_SEARCH_FLAG_IS_OPPED;
                else
                    criteria.flags |= CMD_SEARCH_FLAG_NOT_OPPED;
            } else if(!stricmp(argv[i+1], "voiced")) {
                if(positive)
                    criteria.flags |= CMD_SEARCH_FLAG_IS_VOICED;
                else
                    criteria.flags |= CMD_SEARCH_FLAG_NOT_VOICED;
            }
        }
        else if(!stricmp(argv[i], "limit")) {
            criteria.limit = atoi(argv[i+1]);
        }
    }
    int matches = 0;
    if((criteria.flags & (CMD_SEARCH_FLAG_IS_ZOMBIE | CMD_SEARCH_FLAG_NOT_ZOMBIE))) {
        if(neonserv_cmd_search_zombie) {
            reply(textclient, user, "NS_SEARCH_ZOMBIE_SCAN_IN_PROGRESS");
            return;
        }
        neonserv_cmd_search_zombie = 1;
    }
    
    reply(textclient, user, "NS_SEARCH_HEADER");
    MYSQL_RES *res, *res2;
    MYSQL_ROW row, row2;
    printf_mysql_query("SELECT `channel_name`, `user_user`, `channel_registered`, `channel_nodelete`, `suspended`, `channel_id` FROM `bot_channels` LEFT JOIN `bots` ON `bots`.`id` = `botid` LEFT JOIN `channels` ON `chanid` = `channel_id` LEFT JOIN `users` ON `channel_registrator` = `user_id` WHERE `botclass` = '%d' AND `active` = '1'", client->botid);
    res = mysql_use();
    struct ChanNode *channel;
    while ((row = mysql_fetch_row(res)) != NULL) {
        if((criteria.flags & (CMD_SEARCH_FLAG_IS_ZOMBIE | CMD_SEARCH_FLAG_NOT_ZOMBIE))) {
            channel = getChanByName(row[0]);
            if(channel)
                channel->flags |= CHANFLAG_SCRIPTFLAG1;
            if(show_chans && matches == criteria.limit) {
                //too many
                continue;
            }
        } else {
            if(show_chans && matches == criteria.limit) {
                //too many
                break;
            }
        }
        if((criteria.flags & CMD_SEARCH_FLAG_IS_ZOMBIE)) continue;
        if(criteria.name && match(criteria.name, row[0])) continue;
        if(criteria.registrar && row[1] && match(criteria.registrar, row[1])) continue;
        if(criteria.unvisited) {
            printf_mysql_query("SELECT `chanuser_seen` FROM `chanusers` WHERE `chanuser_cid` = '%s' ORDER BY `chanuser_seen` DESC LIMIT 1", row[5]);
            res2 = mysql_use();
            row2 = mysql_fetch_row(res2);
            if(!row2) continue;
            if((time(0) - atoi(row2[0])) < criteria.unvisited) continue;
        }
        if(criteria.registered && (time(0) - atoi(row[2])) < criteria.registered) continue;
        
        if((criteria.flags & CMD_SEARCH_FLAG_HAS_NODELETE) && strcmp(row[3], "1")) continue;
        if((criteria.flags & CMD_SEARCH_FLAG_NOT_NODELETE) && strcmp(row[3], "0")) continue;
        if((criteria.flags & CMD_SEARCH_FLAG_HAS_SUSPENDED) && strcmp(row[4], "1")) continue;
        if((criteria.flags & CMD_SEARCH_FLAG_NOT_SUSPENDED) && strcmp(row[4], "0")) continue;
        if((criteria.flags & CMD_SEARCH_FLAG_STATES) || criteria.onchan) {
            if(!(criteria.flags & (CMD_SEARCH_FLAG_IS_ZOMBIE | CMD_SEARCH_FLAG_NOT_ZOMBIE)))
                channel = getChanByName(row[0]);
            if(criteria.onchan) {
                if(!channel) continue;
                struct ChanUser *chanuser = NULL;
                for(chanuser = getChannelUsers(channel, NULL); chanuser; chanuser = getChannelUsers(channel, chanuser)) {
                    if(!match(criteria.onchan, chanuser->user->nick)) break;
                }
                if(!chanuser) continue;
            }
            if((criteria.flags & CMD_SEARCH_FLAG_IS_JOINED) && !channel) continue;
            if((criteria.flags & CMD_SEARCH_FLAG_NOT_JOINED) && channel) continue;
            if(channel && (criteria.flags & (CMD_SEARCH_FLAG_IS_OPPED | CMD_SEARCH_FLAG_NOT_OPPED | CMD_SEARCH_FLAG_IS_VOICED | CMD_SEARCH_FLAG_NOT_VOICED))) {
                int flags = 0;
                struct ClientSocket *bot;
                for(bot = getBots(SOCKET_FLAG_READY, NULL); bot; bot = getBots(SOCKET_FLAG_READY, bot)) {
                    if(bot->botid == client->botid) {
                        struct ChanUser *chanuser = getChanUser(bot->user, channel);
                        if(chanuser) {
                            flags = chanuser->flags;
                            break;
                        }
                    }
                }
                if((criteria.flags & CMD_SEARCH_FLAG_IS_OPPED) && !(flags & CHANUSERFLAG_OPPED)) continue;
                if((criteria.flags & CMD_SEARCH_FLAG_NOT_OPPED) && (flags & CHANUSERFLAG_OPPED)) continue;
                if((criteria.flags & CMD_SEARCH_FLAG_IS_VOICED) && !(flags & CHANUSERFLAG_VOICED)) continue;
                if((criteria.flags & CMD_SEARCH_FLAG_NOT_VOICED) && (flags & CHANUSERFLAG_VOICED)) continue;
            }
        }
        matches++;
        //output
        if(show_chans) {
            reply(textclient, user, "%s", row[0]);
        }
    }
    if((criteria.flags & (CMD_SEARCH_FLAG_IS_ZOMBIE | CMD_SEARCH_FLAG_NOT_ZOMBIE))) {
        struct ChanUser *chanuser;
        for(chanuser = getUserChannels(client->user, NULL); chanuser; chanuser = getUserChannels(client->user, chanuser)) {
            channel = chanuser->chan;
            if(channel->flags & CHANFLAG_SCRIPTFLAG1) {
                channel->flags &= ~CHANFLAG_SCRIPTFLAG1;
            } else if((criteria.flags & CMD_SEARCH_FLAG_IS_ZOMBIE)) {
                matches++;
                if(show_chans) {
                    reply(textclient, user, "%s", channel->name);
                }
            }
        }
    }
    neonserv_cmd_search_zombie = 0;
    reply(textclient, user, "NS_TABLE_COUNT", matches);
}
