/* overall.h - NeonServ v5.6
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */

#ifndef _overall_h
#define _overall_h
#include "../config.h"

#define NEONSERV_VERSION "5.6"
#define VERSION_PATCHLEVEL 736

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#ifdef WIN32
#include <windows.h>
#include <winsock2.h>
#include <malloc.h>
#else
#ifdef HAVE_FEATURES_H
#include <features.h>
#endif
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/wait.h>
#include <errno.h>
#endif
#include <unistd.h>
#include <getopt.h>
#include <stdarg.h>
#include <sys/time.h>
#include <time.h>
#include <signal.h>
#ifdef SYNCHRONIZE
//some winnt machines do already have a SYNCHRONIZE macro defined...
#undef SYNCHRONIZE
#endif
#ifdef HAVE_THREADS
#include <pthread.h>
#ifdef WIN32
#define pthread_self_tid() pthread_self().p
#else
#define pthread_self_tid() pthread_self()
#endif
#ifdef PTHREAD_MUTEX_RECURSIVE_NP
#define PTHREAD_MUTEX_RECURSIVE_VAL PTHREAD_MUTEX_RECURSIVE_NP
#else
#define PTHREAD_MUTEX_RECURSIVE_VAL PTHREAD_MUTEX_RECURSIVE
#endif
#define THREAD_MUTEX_INIT(var) { \
    pthread_mutexattr_t mutex_attr; \
    pthread_mutexattr_init(&mutex_attr);\
    pthread_mutexattr_settype(&mutex_attr, PTHREAD_MUTEX_RECURSIVE_VAL);\
    pthread_mutex_init(&var, &mutex_attr); \
}
#define THREAD_MUTEX_INIT_TYPE(var, type) { \
    pthread_mutexattr_t mutex_attr; \
    pthread_mutexattr_init(&mutex_attr);\
    pthread_mutexattr_settype(&mutex_attr, type);\
    pthread_mutex_init(&var, &mutex_attr); \
}
#ifdef ENABLE_MUTEX_DEBUG
#include "mutexDebug.h"
#define SYNCHRONIZE(var) xmutex(1, &var, __FILE__, __LINE__); pthread_mutex_lock(&var)
#define DESYNCHRONIZE(var) xmutex(0, &var, __FILE__, __LINE__); pthread_mutex_unlock(&var)
#else
#define SYNCHRONIZE(var) pthread_mutex_lock(&var)
#define DESYNCHRONIZE(var) pthread_mutex_unlock(&var)
#endif
#else
#define THREAD_MUTEX_INIT(var)
#define SYNCHRONIZE(var)
#define DESYNCHRONIZE(var)
#ifdef ENABLE_MUTEX_DEBUG
#undef ENABLE_MUTEX_DEBUG
#endif
#endif

#if __GNUC__
#define PRINTF_LIKE(M,N) __attribute__((format (printf, M, N)))
#else
#define PRINTF_LIKE(M,N)
#endif

#if __GNUC__ >= 2
#define UNUSED_ARG(ARG) ARG __attribute__((unused))
#elif defined(S_SPLINT_S)
#define UNUSED_ARG(ARG) /*@unused@*/ ARG
#define const /*@observer@*/ /*@temp@*/
#else
#define UNUSED_ARG(ARG) ARG
#endif

#define STRINGIFY_(x) #x
#define STRINGIFY(x) STRINGIFY_(x)
 
#if defined(__GNUC__)
#if defined(__GNUC_PATCHLEVEL__)
#define COMPILER "GCC" " " STRINGIFY(__GNUC__) "." STRINGIFY(__GNUC_MINOR__) "." STRINGIFY(__GNUC_PATCHLEVEL__)
#else
#define COMPILER "GCC" " " STRINGIFY(__GNUC__) "." STRINGIFY(__GNUC_MINOR__)
#endif
#elif defined (__IMAGECRAFT__)
#define COMPILER "ICCAVR"
#else
#define COMPILER "Unknown"
#endif

#ifdef ENABLE_MEMORY_DEBUG
#include "memoryDebug.h"
#endif

#define SOCKET_SELECT_TIME    1
#define SOCKET_RECONNECT_TIME 20

#define NICKLEN         30
#define USERLEN         10
#define AUTHLEN         32
#define HOSTLEN         63
#define REALLEN         50
#define TOPICLEN        500
#define CHANNELLEN      200
#define MAXLEN          512
#define MAXLOGLEN       1024
#define TRIGGERLEN      50
#define MAXNUMPARAMS    200 /* maximum number of parameters in one line */
#define MAXLANGUAGES    5
#define MAXMODES        6
#define INVITE_TIMEOUT  30
#define BOTWAR_DETECTION_TIME 7
#define BOTWAR_DETECTION_EVENTS 6
#define REWHO_TIMEOUT   10 /* wait 10 seconds before WHO an unauthed user again */
#define CLEAR_CACHE_INTERVAL 10

//valid nick chars
#define VALID_NICK_CHARS "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890{|}~[\\]^-_`"
//the first char is a little bit different
//                              0        1         2         3         4         5          6
//                              1234567890123456789012345678901234567890123456789012345678 9012   62
#define VALID_NICK_CHARS_FIRST "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ{|}~[\\]^_`"
#define VALID_NICK_CHARS_FIRST_LEN 62

#define TEMPUSER_LIST_INDEX VALID_NICK_CHARS_FIRST_LEN

#define LOGLEVEL_INFO  0x01
#define LOGLEVEL_ERROR 0x02
#define LOGLEVEL_RAW   0x04
#define LOGLEVEL_MYSQL 0x08

#define timeval_is_bigger(x,y) ((x.tv_sec > y.tv_sec) || (x.tv_sec == y.tv_sec && x.tv_usec > y.tv_usec))

#define MODSTATE_RELOAD    0x01
#define MODSTATE_STARTSTOP 0x02
#define MODSTATE_REBIND    0x03

#endif
